#!/bin/bash

ORIG_DIR=`pwd`
BUILD_DIR=${ORIG_DIR}/work
INSTALL_DIR=${ORIG_DIR}/sut
CC=cc
MAKE=make

cd "${ORIG_DIR}"
if [ -d "${BUILD_DIR}" ]
then
	rm -rf "${BUILD_DIR}"
fi
mkdir "${BUILD_DIR}"
if [ -d "${INSTALL_DIR}" ]
then
	rm -rf "${INSTALL_DIR}"
fi
mkdir "${INSTALL_DIR}"

if [ ! -e "${ORIG_DIR}/openser-1.1.0-notls_src.tar.gz" ]
then
	echo "Trying to download openser-1.1.0-notls_src.tar.gz..."
	wget http://www.openser.org/pub/openser/1.1.0/src/openser-1.1.0-notls_src.tar.gz
fi
if [ ! -e "${ORIG_DIR}/demo.cfg" ]
then
        echo "demo.cfg must be in the same directory as this script."
	exit 1
fi
if [ ! -e "${ORIG_DIR}/openser-1.1.0-notls.patch" ]
then
        echo "openser-1.1.0-notls.patch must be in the same directory as this script."
	exit 1
fi
if [ ! -e "${ORIG_DIR}/ml.c" ] || [ ! -e "${ORIG_DIR}/iters.c" ]
then
        echo "ml.c and iters.c must be in the same directory as this script."
	exit 1
fi
if [ ! -e "${ORIG_DIR}/alog.c" ]
then
        echo "alog.c must be in the same directory as this script."
        exit 1
fi

cd "${BUILD_DIR}"
tar xzf "${ORIG_DIR}/openser-1.1.0-notls_src.tar.gz"
cd openser-1.1.0-notls
patch -p1 < "${ORIG_DIR}/openser-1.1.0-notls.patch"
cd ..
cp "${ORIG_DIR}/ml.c" "${ORIG_DIR}/iters.c" "${BUILD_DIR}"
cp "${ORIG_DIR}/alog.c" "${BUILD_DIR}"
echo "Unpacked sources."

cd openser-1.1.0-notls
"${MAKE}" prefix="${INSTALL_DIR}" all
"${MAKE}" prefix="${INSTALL_DIR}" install
cd ..
if [ ! -e "${INSTALL_DIR}/lib" ] && [ -e "${INSTALL_DIR}/lib64" ]
then
ln -s "${INSTALL_DIR}/lib64" "${INSTALL_DIR}/lib"
fi
sed -e "s#/usr/local#${INSTALL_DIR}#g" < "${ORIG_DIR}/demo.cfg" > "${INSTALL_DIR}/etc/openser/demo.cfg"

"${CC}" -lm -o iters iters.c
cp iters "${ORIG_DIR}"

"${CC}" -lm -o ml ml.c
cp ml "${ORIG_DIR}"

"${CC}" -g -O2 -o alog alog.c
cp alog "${ORIG_DIR}"

cd "${ORIG_DIR}"
rm -rf "${BUILD_DIR}"

