#!/bin/bash

SUTIP=$1
SIPP_REV=57
ORIG_DIR=`pwd`
BUILD_DIR=${ORIG_DIR}/work
CC=cc
MAKE=make

: ${1?"Usage: $0 <SUTIP>"}

cd "${ORIG_DIR}"
if [ -d "${BUILD_DIR}" ]
then
	rm -rf "${BUILD_DIR}"
fi
mkdir "${BUILD_DIR}"

if [ ! -e "${ORIG_DIR}/libosip2-2.2.2.tar.gz" ]
then
	echo "Trying to download libosip2-2.2.2.tar.gz..."
	wget http://ftp.gnu.org/gnu/osip/libosip2-2.2.2.tar.gz
fi
if [ ! -e "${ORIG_DIR}/libeXosip2-2.2.3.tar.gz" ]
then
        echo "Trying to download libeXosip2-2.2.3.tar.gz..."
	wget http://download.savannah.nongnu.org/releases/exosip/libeXosip2-2.2.3.tar.gz
fi
if [ ! -e "${ORIG_DIR}/libosip2-2.2.2.tar.gz" ]
then
	echo "Download libosip2-2.2.2.tar.gz!"
	exit 1
fi
if [ ! -e "${ORIG_DIR}/libeXosip2-2.2.3.tar.gz" ]
then
        echo "Download libeXosip2-2.2.3.tar.gz!"
	exit 1
fi
if [ ! -e "${ORIG_DIR}/sipp-svn-${SIPP_REV}.tar.bz2" ]
then
        echo "Trying to download SIPp ${SIPP_REV}..."
	cd "${BUILD_DIR}"
	svn co -r ${SIPP_REV} https://svn.sourceforge.net/svnroot/sipp/sipp/trunk sipp
	cd "${ORIG_DIR}"
fi
if [ ! -e "${ORIG_DIR}/sip_reg.c" ]
then
        echo "sip_reg.c must be in the same directory as this script."
	exit 1
fi

cd "${BUILD_DIR}"
cp "${ORIG_DIR}/sip_reg.c" "${BUILD_DIR}"
tar xzf "${ORIG_DIR}/libosip2-2.2.2.tar.gz"
tar xzf "${ORIG_DIR}/libeXosip2-2.2.3.tar.gz"
if [ -e "${ORIG_DIR}/sipp-svn-${SIPP_REV}.tar.bz2" ]
then
	tar xjf "${ORIG_DIR}/sipp-svn-${SIPP_REV}.tar.bz2"
	mv sipp-svn-${SIPP_REV} sipp
fi
echo "Unpacked sources."

cd libosip2-2.2.2
./configure --enable-static --prefix="${BUILD_DIR}"
"${MAKE}" install
cd ..
cd libeXosip2-2.2.3
cd src
sed -i s/217.12.3.11/${SUTIP}/g eXutils.c
cd ..
./configure --enable-static --disable-josua --prefix="${BUILD_DIR}"
"${MAKE}" install
cd ..
"${CC}" -DENABLE_DEBUG -I"${BUILD_DIR}/include" -L"${BUILD_DIR}/lib" -o sip_reg sip_reg.c "${BUILD_DIR}/lib/libeXosip2.a" "${BUILD_DIR}/lib/libosip2.a" "${BUILD_DIR}/lib/libosipparser2.a" -lpthread
cp sip_reg "${ORIG_DIR}"

cd sipp
"${MAKE}"
cp sipp "${ORIG_DIR}"
cd ..

cd "${ORIG_DIR}"
rm -rf "${BUILD_DIR}"

