
/* Based on the code from lmbench */


#include <getopt.h>
#include <math.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define tvdiff(tv1, tv2) (((tv2.tv_sec - tv1.tv_sec) * 1000000L) + \
			(tv2.tv_usec - tv1.tv_usec))

static int nbytes = 524288;
static int iter_per_10msec = 0;
int *buf;

void work(int iterations)
{
	register int *lastone = (int *)((char *)buf + nbytes - 512);
	register int sum = 0;

	while (iterations-- > 0) {
		register int *p = buf;
		while (p <= lastone) {
#define DOIT(i) sum += p[i]; p[i] = 1;
			DOIT(0)   DOIT(4)   DOIT(8)   DOIT(12)  DOIT(16)
			DOIT(20)  DOIT(24)  DOIT(28)  DOIT(32)  DOIT(36)
			DOIT(40)  DOIT(44)  DOIT(48)  DOIT(52)  DOIT(56)
			DOIT(60)  DOIT(64)  DOIT(68)  DOIT(72)  DOIT(76)
			DOIT(80)  DOIT(84)  DOIT(88)  DOIT(92)  DOIT(96)
			DOIT(100) DOIT(104) DOIT(108) DOIT(112) DOIT(116)
			DOIT(120) DOIT(124)
			p += 128;
		}
	}
	srand(sum);
}

int main(int argc, char **argv)
{
	struct timeval start, end;
	pid_t kidpid;
	float iterations;
	long duration;
	int verbose = 0;

	int c = getopt(argc, argv, "v");
	if (c != -1)
		if (c == 'v')
			verbose = 1;
		else {
			fprintf(stderr, "Usage: calibrate [-v]\n");
			return 1;
		}

	buf = (int *)valloc(nbytes);
	if (!buf) {
		perror("valloc");
		return 1;
	}

	if (verbose) fprintf(stderr, "(Calibrating.)\n");
	gettimeofday(&start, NULL);
	work(1000000);
	gettimeofday(&end, NULL);

	duration = tvdiff(start, end);
	if (verbose) fprintf(stderr, "(Duration: %lu us)\n", duration);

	iterations = (1000000. / (duration / 10000.));
	if (verbose) fprintf(stderr, "(%.3f iterations ~= 10 ms)\n", iterations);
	iter_per_10msec = (long) rintf(iterations);
	if (verbose)
		fprintf(stderr, "(Use %lu iterations for %.3f ms interval)\n",
		iter_per_10msec,
		iter_per_10msec * (duration / 1000000.) / 1000.);
	fprintf(stdout, "%lu\n", iter_per_10msec);

	return 0;
}
