
/* Based on the code from lmbench */


#include <math.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define tvdiff(tv1, tv2) (((tv2.tv_sec - tv1.tv_sec) * 1000000L) + \
			(tv2.tv_usec - tv1.tv_usec))

static int nbytes = 524288;
int *buf;

void work(int iterations)
{
	register int *lastone = (int *)((char *)buf + nbytes - 512);
	register int sum = 0;

	while (iterations-- > 0) {
		register int *p = buf;
		while (p <= lastone) {
#define DOIT(i) sum += p[i]; p[i] = 1;
			DOIT(0)   DOIT(4)   DOIT(8)   DOIT(12)  DOIT(16)
			DOIT(20)  DOIT(24)  DOIT(28)  DOIT(32)  DOIT(36)
			DOIT(40)  DOIT(44)  DOIT(48)  DOIT(52)  DOIT(56)
			DOIT(60)  DOIT(64)  DOIT(68)  DOIT(72)  DOIT(76)
			DOIT(80)  DOIT(84)  DOIT(88)  DOIT(92)  DOIT(96)
			DOIT(100) DOIT(104) DOIT(108) DOIT(112) DOIT(116)
			DOIT(120) DOIT(124)
			p += 128;
		}
	}
	srand(sum);
}

void do_work(int load, int iterations) {
	int x;

	fprintf(stderr, "(Working ...)\n");
	for (x = 0; x < load; x++)
		work(iterations);
	_exit(0);
}

int main(int argc, char **argv)
{
	struct timeval start, end;
	pid_t kidpid;
	int n = 900, iterations = 0, verbose = 0, load = 0;
	int c;
	while (1) {
		c = getopt(argc, argv, "i:l:n:v");
		if (c == -1) break;
		switch(c) {
		case 'i':	iterations = atoi(optarg); break;
		case 'l':	load = atoi(optarg); break;
		case 'n':	n = atoi(optarg); break;
		case 'v':	verbose = 1; break;
		}
	}

	if (iterations == 0 || load == 0) {
		fprintf(stderr, "Usage: ml -i iterations -l load [-n seconds] [-v]\n");
		return 1;
	}

	if (load > 100)
		if (load == 101)
			fprintf(stderr, "Load @101%% unlikely to work.\n");
		else {
			fprintf(stderr, "Load >100%% capped @100%%.\n");
			load = 100;
		}
	if (load % 10 != 0)
		fprintf(stderr, "Accuracy probably less than 10 %%-units.\n");

	buf = (int *)valloc(nbytes);
	if (!buf) {
		perror("valloc");
		return 1;
	}

	if (verbose) fprintf(stderr, "(%lu iterations)\n", iterations);
	if (verbose) fprintf(stderr, "(CPU load at %d%%)\n", load);
	if (verbose) fprintf(stderr, "(Run for %d seconds)\n", n);

	while (n-- > 0) {
		kidpid = fork();
		if (kidpid == -1) {
			perror("fork");
			exit(1);
		}
		if (kidpid == 0) do_work(load, iterations);
		sleep(1);
		wait(NULL);
	}

	printf("DONE\n");

	return 0;
}
